package com.zhili.zkstationserver.component;

import com.zhili.zkstationserver.configuration.ServerConfig;
import com.zhili.zkstationserver.handler.StationMessageDecoder;
import com.zhili.zkstationserver.handler.StationMessageEncoder;
import com.zhili.zkstationserver.handler.StationMessageHandler;
import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.ChannelOption;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.stereotype.Component;

/**
 * @author ：HuangBin
 * @description：TODO
 * @date ：2022/12/20 15:28
 */
@Slf4j
@Component
@EnableAsync
public class StationServer {
    @Autowired
    ServerConfig serverConfig;

    @Async
    public void run() {
        log.info("serverConfig:" + serverConfig);
        EventLoopGroup bossGroup = new NioEventLoopGroup(1);
        EventLoopGroup workerGroup = new NioEventLoopGroup();
        try {
            ServerBootstrap server = new ServerBootstrap();
            server.group(bossGroup, workerGroup)
                    .channel(NioServerSocketChannel.class)
                    .childOption(ChannelOption.SO_KEEPALIVE, true)
                    .childOption(ChannelOption.TCP_NODELAY, true)
                    .option(ChannelOption.SO_BACKLOG, 1024)
                    .childHandler(new ChannelInitializer<SocketChannel>() {
                        @Override
                        protected void initChannel(SocketChannel socketChannel) throws Exception {
                            socketChannel.pipeline()
                                    .addLast(new StationMessageDecoder())
                                    .addLast(new StationMessageEncoder())
                                    .addLast(new StationMessageHandler());
                        }
                    });
            ChannelFuture channelFuture = server.bind(serverConfig.getPort()).sync();
            log.info("station server running({})", serverConfig.getPort());
            channelFuture.channel().closeFuture().sync();
        } catch (InterruptedException e) {
            e.printStackTrace();
        } finally {
            bossGroup.shutdownGracefully();
            workerGroup.shutdownGracefully();
        }
    }
}
