package com.zhili.zkstationserver.configuration;


import com.alibaba.fastjson.JSON;
import com.zhili.zkstationserver.component.StationServer;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.curator.framework.CuratorFramework;
import org.apache.curator.framework.CuratorFrameworkFactory;
import org.apache.curator.framework.recipes.cache.NodeCacheListener;
import org.apache.curator.framework.recipes.cache.PathChildrenCache;
import org.apache.curator.framework.recipes.cache.PathChildrenCacheEvent;
import org.apache.curator.framework.recipes.cache.PathChildrenCacheListener;
import org.apache.curator.framework.state.ConnectionState;
import org.apache.curator.framework.state.ConnectionStateListener;
import org.apache.curator.retry.RetryForever;
import org.apache.zookeeper.CreateMode;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.nio.charset.StandardCharsets;

import static org.apache.curator.framework.recipes.cache.PathChildrenCacheEvent.Type.*;

/**
 * @author ：HuangBin
 * @description：TODO
 * @date ：2023/1/3 17:27
 */
@Configuration
@Slf4j
public class ZookeeperConfig {
    @Value("${zookeeper.server}")
    String zkAddress;

    @Autowired
    ServerConfig serverConfig;

    @Bean
    public CuratorFramework curatorFramework() throws Exception {
        log.info("serverConfig:{}", serverConfig);
        CuratorFramework client = CuratorFrameworkFactory.builder()
                .connectString(zkAddress)
                .sessionTimeoutMs(5000)
                .connectionTimeoutMs(5000)
                .retryPolicy(new RetryForever(5000)).build();
        //重连的时候都创建已有的临时节点。
        client.getConnectionStateListenable().addListener(new ConnectionStateListener() {
            @SneakyThrows
            @Override
            public void stateChanged(CuratorFramework cli, ConnectionState state) {
                switch (state){
                    case RECONNECTED:
                        log.info("zk RECONNECTED");
                        String instanceNode = serverConfig.getZkRoot() + "/" + serverConfig.getServiceName() + "/"+serverConfig.getInstanceId();
                        cli.create().creatingParentsIfNeeded()
                                .withMode(CreateMode.EPHEMERAL)
                                .forPath(instanceNode, JSON.toJSONString(serverConfig).getBytes(StandardCharsets.UTF_8));
                        break;
                    default:
                        log.info("zk conn other state:"+state.name());
                        break;
                }
            }
        });
        client.start();
        client.blockUntilConnected();
        log.info("zk connected");

         //创建自身站控服务器节点
        String createInstancePath = serverConfig.getZkRoot() + "/" + serverConfig.getServiceName() + "/" + serverConfig.getServiceName() + "-";
        String instanceNode = client.create()
                .creatingParentsIfNeeded()
                .withMode(CreateMode.EPHEMERAL_SEQUENTIAL)
                .forPath(createInstancePath, JSON.toJSONString(serverConfig).getBytes(StandardCharsets.UTF_8));
        log.info("instanceNode:" + instanceNode);
        String instanceId = instanceNode.substring(instanceNode.lastIndexOf("/") + 1);
        serverConfig.setInstanceId(instanceId);
        serverConfig.setTopics(new String[]{instanceId});
        return client;
    }
}
