package com.zhili.zkstationserver.dto;

import com.zhili.zkstationserver.util.BytesUtil;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.apache.commons.lang.ArrayUtils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * @author ：HuangBin
 * @description：TODO
 * @date ：2022/12/21 18:26
 */
@Data
@NoArgsConstructor
public class ExchangeEvent {
    String stationCode;
    String exchangeNo;
    String serialNo;
    Integer seq;
    Integer status;
    String plate;
    String vin;
    String model;
    String sourceSn;
    Float sourceRatedEnergy;
    Float sourceSoc;
    Float sourceSoh;
    Float sourceVol;
    Float sourceCur;
    Integer sourceTotalMiletage;
    Integer targetIndex;
    String targetSn;
    Float targetRatedEnergy;
    Float targetSoc;
    Float targetSoh;
    Float frontDistance;
    Float batteryHeight;
    String timeStr;
    Integer step;
    Integer startSource;
    Date time;
    Integer powerState;
    Integer lockState;

    public class ExchangeSeq {
        public static final int PLATE_RECOGNIZE = 0;
        public static final int VEH_CONNECT = 1;
        public static final int CHOOSE_TARGET_BATTERY = 2;
        public static final int VEH_INFO_VERIFY = 3;
        public static final int VEH_MODEL_VERIFY = 4;
        public static final int READ_SOURCE_BATTERY = 5;
        public static final int SOURCE_BATTERY_UNLOCK = 6;
        public static final int TARGET_BATTERY_READY = 7;
        public static final int SERVO_VERIFY = 8;
        public static final int ROBOT_READY = 9;

        public static final int EXCHANGE_BEGIN = 10;
        public static final int ROBOT_ONGOING = 11;
        public static final int EXCHANGE_END = 12;
    }

    public ExchangeEvent(String stationCode, byte[] data) {
        this.stationCode = stationCode;
        exchangeNo = BytesUtil.parseString(ArrayUtils.subarray(data, 0, 32));
        serialNo = BytesUtil.parseString(ArrayUtils.subarray(data, 32, 64));
        seq = Byte.toUnsignedInt(data[64]);
        status = Byte.toUnsignedInt(data[65]);
        plate = BytesUtil.deUnicode(ArrayUtils.subarray(data, 66, 86));
        vin = BytesUtil.parseString(ArrayUtils.subarray(data, 86, 103));
        model = BytesUtil.deUnicode(ArrayUtils.subarray(data, 103, 119));
        sourceSn = BytesUtil.parseStringZero(ArrayUtils.subarray(data, 119, 146));
        sourceRatedEnergy = BytesUtil.toIntWithLowerFirst(ArrayUtils.subarray(data, 146, 148)) * 0.1f;
        sourceSoc = Byte.toUnsignedInt(data[148]) * 0.4f;
        sourceSoh = Byte.toUnsignedInt(data[149]) * 0.4f;
        sourceVol = BytesUtil.toIntWithLowerFirst(ArrayUtils.subarray(data, 150, 152)) * 0.1f;
        sourceCur = BytesUtil.toIntWithLowerFirst(ArrayUtils.subarray(data, 152, 154)) * 0.1f - 1000f;
        sourceTotalMiletage = BytesUtil.toIntWithLowerFirst(ArrayUtils.subarray(data, 154, 158));
        targetIndex = Byte.toUnsignedInt(data[158]);
        targetSn = BytesUtil.parseString(ArrayUtils.subarray(data, 159, 186));
        targetRatedEnergy = BytesUtil.toIntWithLowerFirst(ArrayUtils.subarray(data, 186, 188)) * 0.1f;
        targetSoc = Byte.toUnsignedInt(data[188]) * 0.4f;
        targetSoh = Byte.toUnsignedInt(data[189]) * 0.4f;
        frontDistance = BytesUtil.toIntWithLowerFirst(ArrayUtils.subarray(data, 190, 194)) * 0.0001f;
        batteryHeight = BytesUtil.toIntWithLowerFirst(ArrayUtils.subarray(data, 194, 198)) * 0.0001f;
        timeStr = BytesUtil.composeTimeString(ArrayUtils.subarray(data, 198, 206));
        step = Byte.toUnsignedInt(data[206]);
        startSource = Byte.toUnsignedInt(data[207]);
        powerState = Byte.toUnsignedInt(data[208]);
        lockState = Byte.toUnsignedInt(data[209]);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date timeD = null;
        try {
            timeD = sdf.parse(timeStr);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        time = timeD;
    }
}
