package com.zhili.zkstationserver.dto;

import com.zhili.zkstationserver.util.BytesUtil;
import lombok.Data;
import org.apache.commons.lang.ArrayUtils;

/**
 * @author ：jek
 * @title 换电记录
 * @date ：2023/2/8 17:34
 */
@Data
public class ExchangeRecordInfo {
    /** 换电编号 */
    String exchangeNo;
    /**	流水号*/
    String serialNo;
    /**	车牌号*/
    String plate;
    /**	vin */
    String vin;
    /**	车型*/
    String model;
    /**	站点编号*/
    String stationCode;
    /**	司机名字*/
    String driverName;
    /** 司机电话  */
    String driverPhone;
    /**  亏电电池sn */
    String sourceBatterySn;
    /**  满电电池sn */
    String targetBatterySn;
    /**  亏电电池容量 */
    float sourceCapacity;
    /** 满电电池容量  */
    float targetCapacity;
    /**  亏电电池soc */
    float sourceSoc;
    /** 满电电池soc  */
    float targetSoc;
    /** 亏电电池soh  */
    float sourceSoh;
    /**  满电电池soh */
    float targetSoh;
    /**  换电量 */
    float exchangePower;
    /**  换电费 */
    float powerFee;
    /** 服务费  */
    float serviceFee;
    /**  开始时间 */
    String startTime;
    /**  结束时间 */
    String endTime;
    /**  发起换电来源 0：小程序，1：站内手动，2：云端*/
    Integer startSource;

    public ExchangeRecordInfo(String stationCode, byte[] data) {
        this.stationCode = stationCode;
        exchangeNo = BytesUtil.parseString(ArrayUtils.subarray(data, 0, 32));
        serialNo = BytesUtil.parseString(ArrayUtils.subarray(data, 32, 64));
        plate = BytesUtil.deUnicode(ArrayUtils.subarray(data, 64, 84));
        vin = BytesUtil.parseString(ArrayUtils.subarray(data, 84, 101));
        model = BytesUtil.deUnicode(ArrayUtils.subarray(data, 101, 117));
        driverName = BytesUtil.deUnicode(ArrayUtils.subarray(data, 149, 181));
        driverPhone = BytesUtil.parseString(ArrayUtils.subarray(data, 181, 192));
        sourceBatterySn = BytesUtil.parseString(ArrayUtils.subarray(data, 192, 219));
        targetBatterySn = BytesUtil.parseString(ArrayUtils.subarray(data, 219, 246));
        sourceCapacity = BytesUtil.toIntWithLowerFirst(ArrayUtils.subarray(data, 246, 248)) * 0.1f;
        targetCapacity = BytesUtil.toIntWithLowerFirst(ArrayUtils.subarray(data, 248, 250)) * 0.1f;
        sourceSoc = Byte.toUnsignedInt(data[250]) * 0.4f;
        targetSoc = Byte.toUnsignedInt(data[251]) * 0.4f;
        sourceSoh = Byte.toUnsignedInt(data[252]) * 0.4f;
        targetSoh = Byte.toUnsignedInt(data[253]) * 0.4f;
        exchangePower = BytesUtil.toIntWithLowerFirst(ArrayUtils.subarray(data, 254, 258)) * 0.4f;
        powerFee = BytesUtil.toIntWithLowerFirst(ArrayUtils.subarray(data, 258, 262)) * 0.2f;
        serviceFee = BytesUtil.toIntWithLowerFirst(ArrayUtils.subarray(data, 262, 266)) * 0.2f;
        startTime = BytesUtil.composeTimeString(ArrayUtils.subarray(data, 266, 274));
        endTime = BytesUtil.composeTimeString(ArrayUtils.subarray(data, 274, 282));
        startSource =  Byte.toUnsignedInt(data[282]);
    }

}
