package com.zhili.zkstationserver.handler;

import com.zhili.zkstationserver.dto.StationMessage;
import com.zhili.zkstationserver.util.BytesUtil;
import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.ByteToMessageDecoder;
import org.apache.commons.lang3.ArrayUtils;

import java.util.List;

/**
 * @author ：HuangBin
 * @description：TODO
 * @date ：2022/12/20 15:40
 */
public class StationMessageDecoder extends ByteToMessageDecoder {
    public static final byte[] START = {(byte) 0xaa, (byte) 0xf5};

    @Override
    protected void decode(ChannelHandlerContext ctx, ByteBuf byteBuf, List<Object> list) throws Exception {
        while (byteBuf.readableBytes() >= START.length) {
            byteBuf.markReaderIndex();
            //寻找头部
            byte[] start = new byte[START.length];
            byteBuf.readBytes(start);
            if (BytesUtil.bytesEqual(start, START)) {
                //如果读到头部，继续解析
                if (byteBuf.readableBytes() >= 2) {
                    //解析长度
                    byte[] lenBytes = new byte[2];
                    byteBuf.readBytes(lenBytes);
                    int len = BytesUtil.toIntWithLowerFirst(lenBytes);
                    if (byteBuf.readableBytes() >= len - 4) {
                        //解析剩下的部分
                        byte[] last = new byte[len - 4];
                        byteBuf.readBytes(last);
                        byte[] checkSumObj = ArrayUtils.subarray(last, 2, last.length - 1);
                        byte b = BytesUtil.sumCheck(checkSumObj);
                        if (b == last[last.length - 1]) {
                            //校验和正确，解析到一个正确的包
                            StationMessage stationMessage = new StationMessage();
                            byte[] commandBytes = ArrayUtils.subarray(last, 2, 4);
                            int cmd = BytesUtil.toIntWithLowerFirst(commandBytes);
                            stationMessage.setCommand(cmd);
                            byte[] dataBytes = ArrayUtils.subarray(last, 4, last.length - 1);
                            stationMessage.setData(dataBytes);
                            list.add(stationMessage);
                        } else {
                            //包不正确，整体丢弃
                        }
                    } else {
                        //读不出一个整体的包，等待数据再读
                        byteBuf.resetReaderIndex();
                        return;
                    }
                } else {
                    //长度字段不够读，下次再读；
                    byteBuf.resetReaderIndex();
                    return;
                }
            } else {
                //没有读到头部，继续往后读
                byteBuf.resetReaderIndex();
                byteBuf.skipBytes(1);
            }
        }
    }
}
